"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const bip39 = __importStar(require("bip39"));
const TezosMessageUtil_1 = require("../../chain/tezos/TezosMessageUtil");
const KeyStore_1 = require("../../types/wallet/KeyStore");
const CryptoUtils_1 = require("../../utils/CryptoUtils");
var TezosWalletUtil;
(function (TezosWalletUtil) {
    function unlockFundraiserIdentity(mnemonic, email, password, pkh) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield getKeysFromMnemonicAndPassphrase(mnemonic, email + password, KeyStore_1.StoreType.Fundraiser, pkh);
        });
    }
    TezosWalletUtil.unlockFundraiserIdentity = unlockFundraiserIdentity;
    function restoreIdentityWithSecretKey(keyString) {
        return __awaiter(this, void 0, void 0, function* () {
            const secretKey = TezosMessageUtil_1.TezosMessageUtils.writeKeyWithHint(keyString, 'edsk');
            const keys = yield CryptoUtils_1.CryptoUtils.recoverPublicKey(secretKey);
            const publicKey = TezosMessageUtil_1.TezosMessageUtils.readKeyWithHint(keys.publicKey, 'edpk');
            const publicKeyHash = TezosMessageUtil_1.TezosMessageUtils.computeKeyHash(keys.publicKey, 'tz1');
            return { publicKey, privateKey: keyString, publicKeyHash, seed: '', storeType: KeyStore_1.StoreType.Mnemonic };
        });
    }
    TezosWalletUtil.restoreIdentityWithSecretKey = restoreIdentityWithSecretKey;
    function generateMnemonic(strength = 256) {
        return bip39.generateMnemonic(strength);
    }
    TezosWalletUtil.generateMnemonic = generateMnemonic;
    function unlockIdentityWithMnemonic(mnemonic, passphrase = '') {
        return __awaiter(this, void 0, void 0, function* () {
            return yield getKeysFromMnemonicAndPassphrase(mnemonic, passphrase, KeyStore_1.StoreType.Mnemonic);
        });
    }
    TezosWalletUtil.unlockIdentityWithMnemonic = unlockIdentityWithMnemonic;
    function getKeysFromMnemonicAndPassphrase(mnemonic, passphrase, storeType, pkh) {
        return __awaiter(this, void 0, void 0, function* () {
            if (![12, 15, 18, 21, 24].includes(mnemonic.split(' ').length)) {
                throw new Error('Invalid mnemonic length.');
            }
            if (!bip39.validateMnemonic(mnemonic)) {
                throw new Error('The given mnemonic could not be validated.');
            }
            const seed = (yield bip39.mnemonicToSeed(mnemonic, passphrase)).slice(0, 32);
            const keys = yield CryptoUtils_1.CryptoUtils.generateKeys(seed);
            const privateKey = TezosMessageUtil_1.TezosMessageUtils.readKeyWithHint(keys.privateKey, 'edsk');
            const publicKey = TezosMessageUtil_1.TezosMessageUtils.readKeyWithHint(keys.publicKey, 'edpk');
            const publicKeyHash = TezosMessageUtil_1.TezosMessageUtils.computeKeyHash(keys.publicKey, 'tz1');
            if (!!pkh && publicKeyHash !== pkh) {
                throw new Error('The given mnemonic and passphrase do not correspond to the applied public key hash');
            }
            return { publicKey, privateKey, publicKeyHash, seed: '', storeType };
        });
    }
    TezosWalletUtil.getKeysFromMnemonicAndPassphrase = getKeysFromMnemonicAndPassphrase;
})(TezosWalletUtil = exports.TezosWalletUtil || (exports.TezosWalletUtil = {}));
//# sourceMappingURL=TezosWalletUtil.js.map